"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const assert_1 = __importDefault(require("assert"));
const upload_1 = require("./upload");
const target_1 = require("./target");
const options_1 = require("./options");
const fs_1 = require("./util/fs");
const log_1 = require("./util/log");
/**
 * 拷贝文件。行为类似 fcp、scp，和 push()、pushMultiple() 的不同在于：
 * 1. source 可以是目录，会递归进去拷贝
 * 2. dest 可以以 / 结尾表示放到目录下面
 *
 * @param sources 本地文件/目录路径列表，基于 cwd 解析
 * @param dest 远程文件路径，为绝对路径
 * @param options 推送参数
 */
async function fcp(sources, dest, raw) {
    const options = options_1.normalize(raw);
    log_1.setLogLevel(options.logLevel);
    log_1.debug('fcp called with', sources, dest, options);
    if (!Array.isArray(sources))
        sources = [sources];
    const copyInto = sources.length > 1 || dest[dest.length - 1] === '/';
    const tasks = [];
    for (const source of sources) {
        const sourceStat = await fs_1.stat(source);
        // source: ./foo/dir or dir/ or dir, dest: /tmp/foo or /tmp/foo/
        if (sourceStat.isDirectory()) {
            assert_1.default(options.recursive, `-r not specified; omitting directory ${source}`);
            for (const file of await fs_1.listFilesRecursively(source.replace(/\/$/, ''))) {
                // file: ./foo/dir/foo.txt or dir/foo.txt
                const destFileName = copyInto ? path_1.join(dest, file) : path_1.join(dest, file.substr(source.length + 1));
                tasks.push({ source: file, dest: destFileName });
            }
        }
        else {
            tasks.push({ source, dest: copyInto ? path_1.join(dest, source) : dest });
        }
    }
    if (options.dryrun) {
        for (const task of tasks) {
            log_1.log(`${task.source} -> ${task.dest}`);
        }
    }
    else {
        return push(tasks, options);
    }
}
exports.fcp = fcp;
/**
 * 推送一组文件
 *
 * @param tasks 一组文件推送任务
 * @param options 推送参数
 */
async function push(tasks, raw) {
    const options = options_1.isNormalizedOptions(raw) ? raw : options_1.normalize(raw);
    log_1.setLogLevel(options.logLevel);
    const upload = new upload_1.Upload(options);
    let successCount = 0;
    let failCount = 0;
    await Promise.all(tasks.map(pushTask));
    if (tasks.length > 1) {
        log_1.success(`total ${tasks.length}, success ${successCount}, fail ${failCount}`);
    }
    async function pushTask({ source, dest }) {
        try {
            await upload.upload(source, dest);
            successCount++;
            log_1.success(source, '>>', dest);
        }
        catch (err) {
            failCount++;
            err.hrmessage = failMessage(err.message, source, dest, options);
            if (options.fastFail)
                throw err;
            log_1.error(err.hrmessage);
        }
    }
}
exports.push = push;
/**
 * 推送单个文件
 *
 * @param source 本地文件路径，基于 cwd 解析
 * @param dest 远程文件路径，为绝对路径
 * @param options 推送参数
 */
async function pushFile(source, dest, raw) {
    const options = options_1.isNormalizedOptions(raw) ? raw : options_1.normalize(raw, { fastFail: true });
    return push([{ source, dest }], options);
}
exports.pushFile = pushFile;
function makit(raw = {}) {
    return function (ctx) {
        const { receiver, dest } = target_1.parseTarget(ctx.target);
        const options = receiver ? options_1.normalize(raw, { receiver, fastFail: true }) : options_1.normalize(raw, { fastFail: true });
        return pushFile(ctx.dependencyFullPath(), dest, options);
    };
}
exports.makit = makit;
function failMessage(message, source, dest, options) {
    return `Upload file "${source}" to "${options.receiver}${dest}" failed: ${message}`;
}
