"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const token_1 = require("./token");
const limited_concurrent_1 = require("./util/limited-concurrent");
const request_1 = require("./util/request");
const promise_1 = require("./util/promise");
const fs_1 = require("./util/fs");
const authenticate_1 = require("./authenticate");
const log_1 = require("./util/log");
const auth = promise_1.singleton(authenticate_1.authenticate);
const endl = '\r\n';
// 每个 receiver 域名下所有任务共享一个队列
const concurrentPool = new Map();
const AUTH_ERR = [100100, 100501, 100101, 100201, 100202, 100302, 100304, 100305];
const RETRY_ERR = [100102, 100307];
class Upload {
    constructor(options) {
        this.options = options;
        this.concurrent = this.createConcurrentInstance();
    }
    createConcurrentInstance() {
        const { receiver, concurrent } = this.options;
        if (!concurrentPool.has(receiver)) {
            concurrentPool.set(receiver, new limited_concurrent_1.LimitedConcurrent(concurrent));
        }
        const instance = concurrentPool.get(receiver);
        instance.setLimit(concurrent);
        return instance;
    }
    async upload(source, dest) {
        return this.concurrent.queue(() => this.uploadFileWithRetry(source, dest));
    }
    async uploadFileWithRetry(src, target, retry = this.options.retry) {
        log_1.debug('uploadWithRetry:', retry);
        try {
            return await this.uploadFile(src, target);
        }
        catch (err) {
            log_1.debug('upload error', err.message);
            // 对于鉴权错误重新鉴权
            if (AUTH_ERR.includes(err.errno)) {
                return auth(this.options).then(() => this.uploadFileWithRetry(src, target, retry));
            }
            // 对于网络错误重试
            if ((RETRY_ERR.includes(err.errno) || !err.errno) && retry > 0) {
                log_1.debug('waiting to retry...');
                return promise_1.wait(100).then(() => this.uploadFileWithRetry(src, target, retry - 1));
            }
            // 其他情况：非网络非鉴权错误（部署路径非法）、重试次数超限
            throw err;
        }
    }
    async uploadFile(path, to) {
        const fileContent = await fs_1.readFile(path);
        const data = { ...await token_1.getToken(), to };
        const boundary = '-----np' + Math.random();
        const collect = [];
        for (const [key, value] of Object.entries(data)) {
            collect.push('--' + boundary + endl);
            collect.push(`Content-Disposition: form-data; name="${key}"`);
            collect.push(endl + endl + value + endl);
        }
        collect.push('--' + boundary + endl);
        collect.push(`Content-Disposition: form-data; name="file"; filename="${path}"`);
        collect.push(endl + endl);
        collect.push(fileContent);
        collect.push(endl);
        collect.push('--' + boundary + '--' + endl);
        return request_1.postFormEncoded(this.options.uploadAPI, boundary, collect);
    }
}
exports.Upload = Upload;
